import time

class Timer:
    def __init__(self):
        self.begin_time = time.time()

    def reset(self):
        self.begin_time = time.time()

    def get_time(self):
        return time.time()-self.begin_time
        


from fastapi.exceptions import RequestValidationError
from starlette.requests import Request
from starlette.responses import JSONResponse
from lantools.exception import SSOException

async def request_validation_exception_handler_422(request: Request, exc: RequestValidationError) -> JSONResponse:
    """
    捕捉422报错并进行自定义处理
    :param request:
    :param exc:
    :return:
    """
    array = []
    for item in exc.errors():
        msg = item['msg']
        field = item['loc'][-1]

        if item['type']=="value_error.missing" and msg=='field required':
            msg = '字段不能为空'
        elif item['type']=='value_error.jsondecode':
            msg = "参数格式错误"
            field = ""

        array.append({
            'field': field,
            'msg': msg

        })

    status_code = 200
    return JSONResponse(
        status_code=status_code,
        content={
            "code": 422,
            "msg": "",
            "data": {
                "detail": array,
                "debug": exc.errors()
            }
        },
    )

async def request_ssoexception_handler(request: Request, exc: SSOException) -> JSONResponse:
    return JSONResponse(
        status_code=200,
        content=exc.output(),
    )


from configparser import ConfigParser

class MyConfig:
    def __init__(self, filename):
        self.parser = ConfigParser()
        self.parser.read(filename)

        self.buffer = self._load()

    def _load(self):
        result = {}
        for section in self.parser.sections():
            temp = {}
            for option in self.parser.options(section):
                name,value=self._get(section, option)
                #print(section, option, name, value)
                temp[name] = value

            result[section] = temp
        return result

    def get(self, section, option=None):
        if option==None:
            return self.buffer[section]
        else:
            return self.buffer[section][option]

    def _get(self, section, option):
        val = self.parser.get(section, option)
        arr = option.split('.')
        #print(option.split('.'))
        if len(arr)==1:
            return arr[0],val
        elif arr[-1]=='array':
            return arr[0],[_convert(item, arr[:-1]) for item in val.split(',')]
        else:
            return arr[0],_convert(val, arr)

def _convert(value, setting):
    if len(setting)==1:
        return value
    else:
        modifier = setting[-1]
        #print(setting, modifier)
        if modifier=='int':
            return int(value)
        elif modifier=='float':
            return float(value)
        elif modifier=='bool':
            return bool(int(value))
        else:
            return value

def read_conf(file):
    return MyConfig(file)



from kafka import KafkaProducer, KafkaConsumer, TopicPartition
from kafka.errors import KafkaTimeoutError
from .mq import Writer, Reader

class KafkaReader(Reader):
    def run(self, callback):
        consumer = KafkaConsumer(
            bootstrap_servers       = self.get_option('bootstrap_servers'),
            group_id                = self.get_option('group_id'),
            enable_auto_commit      = self.get_option('enable_auto_commit', True),
            auto_offset_reset       = self.get_option('auto_offset_reset', 'earliest'),
            max_poll_records        = self.get_option('max_poll_records', 500),
            max_poll_interval_ms    = self.get_option('max_poll_interval_ms', 300000)
        )

        '''auto_offset_reset
        earliest:表示分区下有已提交的offset时，从提交的offset开始消费；无提交的offset时，从头开始消费；
        latest:表示分区下有已提交的offset时，从提交的offset开始消费；无提交的offset时，消费新产生的该分区下的数据
        '''
        consumer.subscribe( self.get_option('topics') )

        for message in consumer:
            callback(message.value, handler=message, consumer=consumer)

class KafkaWriter(Writer):
    def __init__(self, *, bootstrap_servers, topic):
        self.topic = topic
        self.producer = KafkaProducer(bootstrap_servers = bootstrap_servers)

    def write(self, message, *, callback=None):
        def send_success(*args, **kwargs):
            if callback!=None:
                callback(is_success=True)

        def send_error(*args, **kwargs):
            if callback!=None:
                callback(is_success=False)

        try:
            self.producer.send(
                self.topic, 
                message
            ).add_callback(send_success).add_errback(send_error)
        except KafkaTimeoutError as k:
            print("发送超时", k)


from minio import Minio

class PyMinio:
    def __init__(self, host, port, access_key, secret_key, secure=False):
        self.client = Minio(
            "{}:{}".format(host, port),
            access_key=access_key,
            secret_key=secret_key,
            secure=secure
        )

    def set_bucket(self, bucket):
        self.bucket = bucket

    def download(self, object_name:str, local_name:str) -> bool:
        self.client.fget_object(self.bucket, object_name, local_name)
        return True

    def upload(self, file_path:str, object_name:str) -> bool:
        if self.client.bucket_exists(self.bucket) == False:
            self.client.make_bucket(self.bucket)  # 生成一个bucket，类似文件夹

        self.client.fput_object(
            bucket_name=self.bucket,
            object_name=object_name,
            file_path=file_path
        )
        return True

    def exists(self, object_name:str) -> bool:
        try:
            response = self.client.get_object(self.bucket, object_name)
            response.close()
            response.release_conn()
            return True
        except Exception as e:
            return False

    # 删除对象
    def remove_file(self, object_name:str) -> bool:
        try:
            self.client.remove_object(self.bucket, object_name)
            print("Sussess")
            return True
        except Exception as err:
            print(err)
            return False

    def get_client(self) -> Minio:
        return self.client



#-*- encoding: UTF-8 -*-
from setuptools import setup, find_packages

setup(
    name = "lantools",                # 包名
    author = "changwei",
    author_email = "nkchwfree@163.com",
    version = "0.0.17",              # 版本信息
    packages = find_packages("."), #['lantools', 'lantools/decorator', 'lantools/fastapi', 'lantools/mq/', 'lantools/io/'],          # 要打包的项目文件夹
    include_package_data=True,    # 自动打包文件夹内所有数据
    zip_safe=True,                # 设定项目包为安全，不用每次都检测其安全性
    install_requires = [          # 安装依赖的其他包（测试数据）
    ],

    # 设置程序的入口为path
    # 安装后，命令行执行path相当于调用get_path.py中的fun方法
    # entry_points={
    #     'console_scripts':[
    #         'path = demo.get_path:fun'
    #                                   ]
    # },
)



def read_lines(filename, strip=False):
    # 打开文件
    fo = open(filename, "r")
    
    result =[]
    for line in fo.readlines():
        if strip==True:
            line = line.strip()

        result.append(line)
    
    # 关闭文件
    fo.close()

    return result


class SSOException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message

    def output(self):
        return {"code": self.code, "msg": self.message, "data": {}}